@@ -2,14 +2,14 @@
 Dataclass Wizard
 ~~~~~~~~~~~~~~~~
 
-Marshal dataclasses to/from JSON and Python dict objects. Support properties
-with initial values. Generate a dataclass schema for JSON input.
+Lightning-fast JSON wizardry for Python dataclasses — effortless
+serialization right out of the box!
 
 Sample Usage:
 
     >>> from dataclasses import dataclass, field
     >>> from datetime import datetime
-    >>> from typing import Optional, List
+    >>> from typing import Optional
     >>>
     >>> from dataclass_wizard import JSONSerializable, property_wizard
     >>>
@@ -18,7 +18,7 @@ Sample Usage:
     >>> class MyClass(JSONSerializable, metaclass=property_wizard):
     >>>
     >>>     my_str: Optional[str]
-    >>>     list_of_int: List[int] = field(default_factory=list)
+    >>>     list_of_int: list[int] = field(default_factory=list)
     >>>     # You can also define this as `my_dt`, however only the annotation
     >>>     # will carry over in that case, since the value is re-declared by
     >>>     # the property below.
@@ -64,20 +64,23 @@ Sample Usage:
 For full documentation and more advanced usage, please see
 <https://dataclass-wizard.readthedocs.io>.
 
-:copyright: (c) 2021 by Ritvik Nag.
+:copyright: (c) 2021-2025 by Ritvik Nag.
 :license: Apache 2.0, see LICENSE for more details.
 """
 
 __all__ = [
     # Base exports
     'JSONSerializable',
+    'JSONPyWizard',
     'JSONWizard',
     'LoadMixin',
     'DumpMixin',
     'property_wizard',
     # Wizard Mixins
+    'EnvWizard',
     'JSONListWizard',
     'JSONFileWizard',
+    'TOMLWizard',
     'YAMLWizard',
     # Helper serializer functions + meta config
     'fromlist',
@@ -85,35 +88,55 @@ __all__ = [
     'asdict',
     'LoadMeta',
     'DumpMeta',
+    'EnvMeta',
     # Models
+    'env_field',
     'json_field',
     'json_key',
+    'path_field',
+    'skip_if_field',
+    'KeyPath',
     'Container',
     'Pattern',
     'DatePattern',
     'TimePattern',
     'DateTimePattern',
+    'CatchAll',
+    'SkipIf',
+    'SkipIfNone',
+    'EQ',
+    'NE',
+    'LT',
+    'LE',
+    'GT',
+    'GE',
+    'IS',
+    'IS_NOT',
+    'IS_TRUTHY',
+    'IS_FALSY',
 ]
 
 import logging
 
-from .bases_meta import LoadMeta, DumpMeta
-from .constants import PY36
+from .bases_meta import LoadMeta, DumpMeta, EnvMeta
+from .constants import PACKAGE_NAME
 from .dumpers import DumpMixin, setup_default_dumper, asdict
-from .loaders import LoadMixin, setup_default_loader, fromlist, fromdict
-from .models import (json_field, json_key, Container,
-                     Pattern, DatePattern, TimePattern, DateTimePattern)
+from .loaders import LoadMixin, setup_default_loader
+from .loader_selection import fromlist, fromdict
+from .models import (env_field, json_field, json_key, path_field, skip_if_field,
+                     KeyPath, Container,
+                     Pattern, DatePattern, TimePattern, DateTimePattern,
+                     CatchAll, SkipIf, SkipIfNone,
+                     EQ, NE, LT, LE, GT, GE, IS, IS_NOT, IS_TRUTHY, IS_FALSY)
+from .environ.wizard import EnvWizard
 from .property_wizard import property_wizard
-from .serial_json import JSONSerializable
-from .wizard_mixins import JSONListWizard, JSONFileWizard, YAMLWizard
+from .serial_json import JSONWizard, JSONPyWizard, JSONSerializable
+from .wizard_mixins import JSONListWizard, JSONFileWizard, TOMLWizard, YAMLWizard
 
 
 # Set up logging to ``/dev/null`` like a library is supposed to.
 # http://docs.python.org/3.3/howto/logging.html#configuring-logging-for-a-library
-logging.getLogger('dataclass_wizard').addHandler(logging.NullHandler())
-
-# A handy alias in case it comes in useful to anyone :)
-JSONWizard = JSONSerializable
+logging.getLogger(PACKAGE_NAME).addHandler(logging.NullHandler())
 
 # Setup the default type hooks to use when converting `str` (json) or a Python
 # `dict` object to a `dataclass` instance.
@@ -122,10 +145,3 @@ setup_default_loader()
 # Setup the default type hooks to use when converting `dataclass` instances to
 # a JSON `string` or a Python `dict` object.
 setup_default_dumper()
-
-if PY36:    # pragma: no cover
-    # Python 3.6 requires a backport for `datetime.fromisoformat()`
-    # noinspection PyPackageRequirements
-    # noinspection PyUnresolvedReferences
-    from backports.datetime_fromisoformat import MonkeyPatch
-    MonkeyPatch.patch_fromisoformat()
