@@ -8,13 +8,15 @@ obtained from http://www.apache.org/licenses/LICENSE-2.0.
 
 See the end of this file for the original Apache license from this library.
 """
+from base64 import b64encode
 from collections import defaultdict, deque
-# noinspection PyProtectedMember
+# noinspection PyProtectedMember,PyUnresolvedReferences
 from dataclasses import _is_dataclass_instance
 from datetime import datetime, time, date, timedelta
 from decimal import Decimal
 from enum import Enum
-from typing import Type, List, Dict, Any, NamedTupleMeta, Optional, Callable
+# noinspection PyProtectedMember,PyUnresolvedReferences
+from typing import Type, List, Dict, Any, NamedTupleMeta, Optional, Callable, Collection
 from uuid import UUID
 
 from .abstractions import AbstractDumper
@@ -24,16 +26,24 @@ from .class_helper import (
     dataclass_field_names, dataclass_field_to_default,
     dataclass_field_to_json_field,
     dataclass_to_dumper, set_class_dumper,
-    _CLASS_TO_DUMP_FUNC, setup_dump_config_for_cls_if_needed, get_meta,
-    dataclass_field_to_load_parser,
+    CLASS_TO_DUMP_FUNC, setup_dump_config_for_cls_if_needed, get_meta,
+    dataclass_field_to_load_parser, dataclass_field_to_json_path, is_builtin, dataclass_field_to_skip_if,
+    v1_dataclass_field_to_alias,
 )
-from .constants import _DUMP_HOOKS, TAG
+from .constants import _DUMP_HOOKS, TAG, CATCH_ALL
 from .decorators import _alias
+from .errors import show_deprecation_warning
+from .loader_selection import _get_load_fn_for_dataclass
 from .log import LOG
+from .models import get_skip_if_condition, finalize_skip_if
 from .type_def import (
-    ExplicitNull, NoneType, JSONObject,
+    Buffer, ExplicitNull, NoneType, JSONObject,
     DD, LSQ, E, U, LT, NT, T
 )
+from .utils.dict_helper import NestedDict
+from .utils.function_builder import FunctionBuilder
+# noinspection PyProtectedMember
+from .utils.dataclass_compat import _set_new_attribute
 from .utils.string_conv import to_camel_case
 
 
@@ -68,6 +78,10 @@ class DumpMixin(AbstractDumper, BaseDumpHook):
     def dump_with_str(o: str, *_):
         return o
 
+    @staticmethod
+    def dump_with_bytes(o: Buffer, *_) -> str:
+        return b64encode(o).decode()
+
     @staticmethod
     def dump_with_int(o: int, *_):
         return o
@@ -150,8 +164,8 @@ def setup_default_dumper(cls=DumpMixin):
     cls.register_dump_hook(int, cls.dump_with_int)
     cls.register_dump_hook(float, cls.dump_with_float)
     cls.register_dump_hook(bool, cls.dump_with_bool)
-    cls.register_dump_hook(bytes, cls.default_dump_with)
-    cls.register_dump_hook(bytearray, cls.default_dump_with)
+    cls.register_dump_hook(bytes, cls.dump_with_bytes)
+    cls.register_dump_hook(bytearray, cls.dump_with_bytes)
     cls.register_dump_hook(NoneType, cls.dump_with_null)
     # Complex types
     cls.register_dump_hook(Enum, cls.dump_with_enum)
@@ -199,9 +213,11 @@ def get_dumper(cls=None, create=True) -> Type[DumpMixin]:
         return set_class_dumper(cls, DumpMixin)
 
 
-def asdict(obj: T,
-           *, cls=None, dict_factory=dict,
-           exclude: List[str] = None, **kwargs) -> JSONObject:
+def asdict(o: T,
+           *, cls=None,
+           dict_factory=dict,
+           exclude: 'Collection[str] | None' = None,
+           **kwargs) -> JSONObject:
     # noinspection PyUnresolvedReferences
     """Return the fields of a dataclass instance as a new dictionary mapping
     field names to field values.
@@ -234,14 +250,14 @@ def asdict(obj: T,
     # if not _is_dataclass_instance(obj):
     #     raise TypeError("asdict() should be called on dataclass instances")
 
-    cls = cls or type(obj)
+    cls = cls or type(o)
 
     try:
-        dump = _CLASS_TO_DUMP_FUNC[cls]
+        dump = CLASS_TO_DUMP_FUNC[cls]
     except KeyError:
         dump = dump_func_for_dataclass(cls)
 
-    return dump(obj, dict_factory, exclude, **kwargs)
+    return dump(o, dict_factory, exclude, **kwargs)
 
 
 def dump_func_for_dataclass(cls: Type[T],
@@ -249,6 +265,8 @@ def dump_func_for_dataclass(cls: Type[T],
                             nested_cls_to_dump_func: Dict[Type, Any] = None,
                             ) -> Callable[[T, Any, Any, Any], JSONObject]:
 
+    # TODO dynamically generate for multiple nested classes at once
+
     # Get the dumper for the class, or create a new one as needed.
     cls_dumper = get_dumper(cls)
 
@@ -265,17 +283,20 @@ def dump_func_for_dataclass(cls: Type[T],
         if meta.recursive and meta is not AbstractMeta:
             config = meta
 
-    else:  # we are being run for a nested dataclass
-        if config:
-            # we want to apply the meta config from the main dataclass
-            # recursively.
-            meta = meta | config
-            meta.bind_to(cls, is_default=False)
+    # we are being run for a nested dataclass
+    elif config:
+        # we want to apply the meta config from the main dataclass
+        # recursively.
+        meta = meta | config
+        meta.bind_to(cls, is_default=False)
 
     # This contains the dump hooks for the dataclass. If the class
     # sub-classes from `DumpMixIn`, these hooks could be customized.
     hooks = cls_dumper.__DUMP_HOOKS__
 
+    # TODO this is temporary
+    if meta.v1:
+        _ = v1_dataclass_field_to_alias(cls)
     # Set up the initial dump config for the dataclass.
     setup_dump_config_for_cls_if_needed(cls)
 
@@ -288,6 +309,9 @@ def dump_func_for_dataclass(cls: Type[T],
     # via a `default` or `default_factory` argument.
     field_to_default = dataclass_field_to_default(cls)
 
+    # A cached mapping of dataclass field name to its SkipIf condition.
+    field_to_skip_if = dataclass_field_to_skip_if(cls)
+
     # A collection of field names in the dataclass.
     field_names = dataclass_field_names(cls)
 
@@ -297,90 +321,202 @@ def dump_func_for_dataclass(cls: Type[T],
         # we don't process the class annotations here. So instead, generate
         # the load parser for each field  (if needed), but don't cache the
         # result, as it's conceivable we might yet call `LoadMeta` later.
-        from .loaders import get_loader
-        cls_loader = get_loader(cls)
-        # Use the cached result if it exists, but don't cache it ourselves.
-        _ = dataclass_field_to_load_parser(
-            cls_loader, cls, config, save=False)
+        from .loader_selection import get_loader
+
+        if meta.v1:
+            # TODO there must be a better way to do this,
+            #   this is just a temporary workaround.
+            try:
+                _ = _get_load_fn_for_dataclass(cls, v1=True)
+            except Exception:
+                pass
+        else:
+            cls_loader = get_loader(cls, v1=meta.v1)
+            # Use the cached result if it exists, but don't cache it ourselves.
+            _ = dataclass_field_to_load_parser(
+                cls_loader, cls, config, save=False)
 
     # Tag key to populate when a dataclass is in a `Union` with other types.
     tag_key = meta.tag_key or TAG
 
-    def cls_asdict(obj: T, dict_factory=dict,
-                   exclude: List[str] = None,
-                   skip_defaults=meta.skip_defaults) -> JSONObject:
-        """
-        Serialize a dataclass of type `cls` to a Python dictionary object.
-        """
+    catch_all_field = dataclass_to_json_field.get(CATCH_ALL)
+    has_catch_all = catch_all_field is not None
 
-        # Call the optional hook that runs before we process the dataclass
-        cls_dumper.__pre_as_dict__(obj)
+    field_to_path = dataclass_field_to_json_path(cls)
+    num_paths = len(field_to_path)
+    has_json_paths = True if num_paths else False
 
-        # This a list that contains a mapping of each dataclass field to its
-        # serialized value.
-        result = []
+    skip_defaults = True if meta.skip_defaults or meta.skip_defaults_if else False
 
-        # Loop over the dataclass fields
-        for field in field_names:
+    _locals = {
+        'config': config,
+        'asdict': _asdict_inner,
+        'hooks': hooks,
+        'cls_to_asdict': nested_cls_to_dump_func,
+    }
 
-            # Get the resolved JSON field name
-            try:
-                json_field = dataclass_to_json_field[field]
+    _globals = {}
 
-            except KeyError:
-                # Normalize the dataclass field name (by default to camel
-                # case)
-                json_field = cls_dumper.transform_dataclass_field(field)
-                dataclass_to_json_field[field] = json_field
+    skip_if_condition = get_skip_if_condition(
+        meta.skip_if, _locals, '_skip_value')
 
-            # Exclude any dataclass fields that are explicitly ignored.
-            if json_field is ExplicitNull:
-                continue
-            if exclude and field in exclude:
-                continue
+    skip_defaults_if_condition = get_skip_if_condition(
+        meta.skip_defaults_if, _locals, '_skip_defaults_value')
 
-            # -- This line is *mostly* the same as in the original version --
-            fv = getattr(obj, field)
+    # Initialize FuncBuilder
+    fn_gen = FunctionBuilder()
 
-            # Check if we need to strip defaults, and the field currently
-            # is assigned a default value.
-            #
-            # TODO: maybe it makes sense to move this logic to a separate
-            #   function, as it might be slightly more performant.
-            if skip_defaults and field in field_to_default \
-                    and fv == field_to_default[field]:
-                continue
+    # Code for `cls_asdict`
+    with fn_gen.function('cls_asdict',
+                         ['o',
+                          'dict_factory=dict',
+                          "exclude:'list[str]|None'=None",
+                          f'skip_defaults:bool={skip_defaults}'],
+                         'JSONObject',
+                         _locals):
 
-            value = _asdict_inner(fv, dict_factory, hooks, config,
-                                  nested_cls_to_dump_func)
+        if (
+            _pre_dict := getattr(cls, '_pre_dict', None)
+        ) is not None:
+            # class defines a `_pre_dict()`
+            _locals['__pre_dict__'] = _pre_dict
+            fn_gen.add_line('__pre_dict__(o)')
+        elif (
+            _pre_dict := getattr(cls_dumper, '__pre_as_dict__', None)
+        ) is not None:
+            # deprecated since v0.28.0
+            # subclass of `DumpMixin` defines a `__pre_as_dict__()`
+            reason = "use `_pre_dict` instead - no need to subclass from DumpMixin"
+            show_deprecation_warning(_pre_dict, reason)
 
-            # -- This line is *mostly* the same as in the original version --
-            result.append((json_field, value))
+            _locals['__pre_dict__'] = _pre_dict
 
-        # -- This line is the same as in the original version --
-        return dict_factory(result)
+            # Call the optional hook that runs before we process the dataclass
+            fn_gen.add_line('__pre_dict__(o)')
 
-    def cls_asdict_with_tag(obj: T, dict_factory=dict,
-                            exclude: List[str] = None,
-                            **kwargs) -> JSONObject:
-        """
-        Serialize a dataclass of type `cls` to a Python dictionary object.
-        Adds a tag field when `tag` field is passed in Meta.
-        """
-        result = cls_asdict(obj, dict_factory, exclude, **kwargs)
-        result[tag_key] = meta.tag
+        # Initialize result list to hold field mappings
+        fn_gen.add_line("result = []")
 
-        return result
+        if has_json_paths:
+            _locals['NestedDict'] = NestedDict
+            fn_gen.add_line('paths = NestedDict()')
 
-    if meta.tag:
-        asdict_func = cls_asdict_with_tag
-    else:
-        asdict_func = cls_asdict
+        if field_names:
+
+            skip_field_assignments = []
+            exclude_assignments = []
+            skip_default_assignments = []
+            field_assignments = []
+
+            # Loop over the dataclass fields
+            for i, field in enumerate(field_names):
+                skip_field = f'_skip_{i}'
+                skip_if_field = f'_skip_if_{i}'
+                default_value = f'_default_{i}'
+
+                skip_field_assignments.append(skip_field)
+                exclude_assignments.append(
+                    f'{skip_field}={field!r} in exclude'
+                )
+                if field in field_to_default:
+                    if skip_defaults_if_condition:
+                        _final_skip_if = finalize_skip_if(
+                            meta.skip_defaults_if, f'o.{field}', skip_defaults_if_condition)
+                        skip_default_assignments.append(
+                            f"{skip_field} = {skip_field} or {_final_skip_if}"
+                        )
+                    else:
+                        _locals[default_value] = field_to_default[field]
+                        skip_default_assignments.append(
+                            f"{skip_field} = {skip_field} or o.{field} == {default_value}"
+                        )
+
+                # Get the resolved JSON field name
+                try:
+                    json_field = dataclass_to_json_field[field]
+                except KeyError:
+                    # Normalize the dataclass field name (by default to camel
+                    # case)
+                    json_field = cls_dumper.transform_dataclass_field(field)
+                    dataclass_to_json_field[field] = json_field
+
+                # Exclude any dataclass fields that are explicitly ignored.
+                if json_field is not ExplicitNull:
+                    # If field has an explicit `SkipIf` condition
+                    if field in field_to_skip_if:
+                        _skip_condition = field_to_skip_if[field]
+                        _skip_if = get_skip_if_condition(
+                            _skip_condition, _locals, skip_if_field)
+                        _final_skip_if = finalize_skip_if(
+                            _skip_condition, f'o.{field}', _skip_if)
+                        field_assignments.append(f'if not ({skip_field} or {_final_skip_if}):')
+                    # If Meta `skip_if` has a value
+                    elif skip_if_condition:
+                        _final_skip_if = finalize_skip_if(
+                            meta.skip_if, f'o.{field}', skip_if_condition)
+                        field_assignments.append(f'if not ({skip_field} or {_final_skip_if}):')
+                    # Else, proceed as normal
+                    else:
+                        field_assignments.append(f"if not {skip_field}:")
+
+                    if json_field:
+                        field_assignments.append(f"  result.append(('{json_field}',"
+                                                 f"asdict(o.{field},dict_factory,hooks,config,cls_to_asdict)))")
+                    # Empty string, will be the case for a dataclass
+                    # field which specifies a "JSON Path".
+                    else:
+                        path = field_to_path[field]
+                        key_part = ''.join(f'[{p!r}]' for p in path)
+                        field_assignments.append(
+                            f'  paths{key_part} = asdict(o.{field},dict_factory,hooks,config,cls_to_asdict)')
+
+                elif has_catch_all and catch_all_field == field:
+                    if field in field_to_default:
+                        field_assignments.append(f"if o.{field} != {default_value} and not {skip_field}:")
+                    else:
+                        field_assignments.append(f"if not {skip_field}:")
+                    field_assignments.append(f"  for k, v in o.{field}.items():")
+                    field_assignments.append("    result.append((k,"
+                                             "asdict(v,dict_factory,hooks,config,cls_to_asdict)))")
+
+            with fn_gen.if_('exclude is None'):
+                fn_gen.add_line('='.join(skip_field_assignments) + '=False')
+            with fn_gen.else_():
+                fn_gen.add_line(';'.join(exclude_assignments))
+
+            if skip_default_assignments:
+                with fn_gen.if_('skip_defaults'):
+                    fn_gen.add_lines(*skip_default_assignments)
+
+            fn_gen.add_lines(*field_assignments)
+
+        if has_json_paths:
+            fn_gen.add_line("result and paths.update(result); result = paths")
+
+        # Return the final dictionary result
+        if meta.tag:
+            fn_gen.add_line("result = dict_factory(result)")
+            fn_gen.add_line(f"result[{tag_key!r}] = {meta.tag!r}")
+            # Return the result with the tag added
+            fn_gen.add_line("return result")
+        else:
+            fn_gen.add_line("return dict_factory(result)")
+
+    # Compile the code into a dynamic string
+    functions = fn_gen.create_functions(_globals)
+
+    cls_asdict = functions['cls_asdict']
+
+    asdict_func = cls_asdict
 
     # In any case, save the dump function for the class, so we don't need to
     # run this logic each time.
     if is_main_class:
-        _CLASS_TO_DUMP_FUNC[cls] = asdict_func
+        # Check if the class has a `to_dict`, and it's
+        # equivalent to `asdict`.
+        if getattr(cls, 'to_dict', None) is asdict:
+            _set_new_attribute(cls, 'to_dict', asdict_func)
+        CLASS_TO_DUMP_FUNC[cls] = asdict_func
     else:
         nested_cls_to_dump_func[cls] = asdict_func
 
@@ -392,7 +528,9 @@ def dump_func_for_dataclass(cls: Type[T],
 # method has also been heavily modified from the original implementation in
 # `dataclasses`. However, I will call out specific lines where it is taken
 # directly from the original version.
-def _asdict_inner(obj, dict_factory, hooks, meta, cls_to_dump_func) -> Any:
+def _asdict_inner(obj, dict_factory, hooks, meta, cls_to_dump_func,
+                  # Added for `EnvWizard` (environ/dumpers.py)
+                  dump_func_for_cls=dump_func_for_dataclass) -> Any:
 
     cls = type(obj)
     dump_hook = hooks.get(cls)
@@ -405,7 +543,7 @@ def _asdict_inner(obj, dict_factory, hooks, meta, cls_to_dump_func) -> Any:
         try:
             dump = cls_to_dump_func[cls]
         except KeyError:
-            dump = dump_func_for_dataclass(cls, meta, cls_to_dump_func)
+            dump = dump_func_for_cls(cls, meta, cls_to_dump_func)
         # noinspection PyArgumentList
         return dump(obj, dict_factory=dict_factory)
 
