@@ -1,32 +1,42 @@
+import collections.abc as abc
 from collections import defaultdict, deque, namedtuple
-from dataclasses import is_dataclass
+from dataclasses import is_dataclass, MISSING
 from datetime import datetime, time, date, timedelta
 from decimal import Decimal
 from enum import Enum
+from pathlib import Path
+# noinspection PyUnresolvedReferences,PyProtectedMember
 from typing import (
     Any, Type, Dict, List, Tuple, Iterable, Sequence, Union,
-    NamedTupleMeta, SupportsFloat, AnyStr, Text, Callable, Optional
+    NamedTupleMeta,
+    SupportsFloat, AnyStr, Text, Callable, Optional
 )
 from uuid import UUID
 
-from .abstractions import AbstractLoader, AbstractParser, FieldToParser
+from .abstractions import AbstractLoader, AbstractParser
 from .bases import BaseLoadHook, AbstractMeta, META
 from .class_helper import (
-    get_class_name, create_new_class,
-    dataclass_to_loader, set_class_loader,
     dataclass_field_to_load_parser, json_field_to_dataclass_field,
-    _CLASS_TO_LOAD_FUNC, dataclass_fields, get_meta, is_subclass_safe,
+    CLASS_TO_LOAD_FUNC, dataclass_fields, get_meta, is_subclass_safe, dataclass_field_to_json_path,
+    dataclass_init_fields, dataclass_field_to_default,
 )
-from .constants import _LOAD_HOOKS, SINGLE_ARG_ALIAS, IDENTITY
+from .constants import SINGLE_ARG_ALIAS, IDENTITY, CATCH_ALL
 from .decorators import _alias, _single_arg_alias, resolve_alias_func, _identity
-from .errors import ParseError, MissingFields, UnknownJSONKey, MissingData
+from .errors import (ParseError, MissingFields, UnknownKeysError,
+                     MissingData, RecursiveClassError)
+from .loader_selection import fromdict, get_loader
 from .log import LOG
-from .models import Extras, _PatternedDT
+from .models import Extras, PatternedDT
 from .parsers import *
 from .type_def import (
     ExplicitNull, FrozenKeys, DefFactory, NoneType, JSONObject,
+    PyRequired, PyNotRequired,
     M, N, T, E, U, DD, LSQ, NT
 )
+# noinspection PyProtectedMember
+from .utils.dataclass_compat import _set_new_attribute
+from .utils.function_builder import FunctionBuilder
+from .utils.object_path import safe_get
 from .utils.string_conv import to_snake_case
 from .utils.type_conv import (
     as_bool, as_str, as_datetime, as_date, as_time, as_int, as_timedelta
@@ -134,7 +144,7 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
     @staticmethod
     def load_to_named_tuple(
             o: Union[Dict, List, Tuple], base_type: Type[NT],
-            field_to_parser: FieldToParser,
+            field_to_parser: 'FieldToParser',
             field_parsers: List[AbstractParser]) -> NT:
 
         if isinstance(o, dict):
@@ -185,7 +195,7 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
     @staticmethod
     def load_to_typed_dict(
             o: Dict, base_type: Type[M],
-            key_to_parser: FieldToParser,
+            key_to_parser: 'FieldToParser',
             required_keys: FrozenKeys,
             optional_keys: FrozenKeys) -> M:
 
@@ -207,6 +217,11 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
 
         return base_type(str(o))
 
+    @staticmethod
+    def load_to_path(o: N, base_type: Type[Path]) -> Path:
+
+        return base_type(str(o))
+
     @staticmethod
     @_alias(as_datetime)
     def load_to_datetime(
@@ -233,10 +248,19 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
         # alias: as_timedelta
         ...
 
+    @staticmethod
+    def load_func_for_dataclass(
+        cls: Type[T],
+        config: Optional[META],
+    ) -> Callable[[JSONObject], T]:
+
+        return load_func_for_dataclass(
+            cls, is_main_class=False, config=config)
+
     @classmethod
     def get_parser_for_annotation(cls, ann_type: Type[T],
                                   base_cls: Type = None,
-                                  extras: Extras = None) -> AbstractParser:
+                                  extras: Extras = None) -> 'AbstractParser | Callable[[dict[str, Any]], T]':
         """Returns the Parser (dispatcher) for a given annotation type."""
         hooks = cls.__LOAD_HOOKS__
         ann_type = eval_forward_ref_if_needed(ann_type, base_cls)
@@ -276,15 +300,30 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
             # class or a non-generic sub-type.
             except AttributeError:
 
-                if isinstance(base_type, type):
+                # https://stackoverflow.com/questions/76520264/dataclasswizard-after-upgrading-to-python3-11-is-not-working-as-expected
+                if base_type is Any:
+                    load_hook = cls.default_load_to
+
+                elif isinstance(base_type, type):
 
                     if is_dataclass(base_type):
-                        base_type: Type[T]
-                        load_hook = load_func_for_dataclass(
-                            base_type,
-                            is_main_class=False,
-                            config=extras['config']
-                        )
+                        config: META = extras.get('config')
+
+                        # enable support for cyclic / self-referential dataclasses
+                        # see https://github.com/rnag/dataclass-wizard/issues/62
+                        if AbstractMeta.recursive_classes or (config and config.recursive_classes):
+                            # noinspection PyTypeChecker
+                            return RecursionSafeParser(
+                                base_cls, extras, base_type, hook=None
+                            )
+                        else:  # else, logic is same as normal
+                            base_type: 'type[T]'
+                            # return a dynamically generated `fromdict`
+                            # for the `cls` (base_type)
+                            return cls.load_func_for_dataclass(
+                                base_type,
+                                config=extras['config']
+                            )
 
                     elif issubclass(base_type, Enum):
                         load_hook = hooks.get(Enum)
@@ -317,10 +356,7 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
                             cls.get_parser_for_annotation
                         )
 
-                elif base_type is Any:
-                    load_hook = cls.default_load_to
-
-                elif isinstance(base_type, _PatternedDT):
+                elif isinstance(base_type, PatternedDT):
                     # Check for a field that was initially annotated like:
                     #   DateTimePattern('%m/%d/%y %H:%M:%S')]
                     return PatternedDTParser(base_cls, extras, base_type)
@@ -359,6 +395,12 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
                             cls.get_parser_for_annotation
                         )
 
+                elif base_type in (PyRequired, PyNotRequired):
+                    # Given `Required[T]` or `NotRequired[T]`, we only need `T`
+                    ann_type = get_args(ann_type)[0]
+                    return cls.get_parser_for_annotation(
+                        ann_type, base_cls, extras)
+
                 elif issubclass(base_type, defaultdict):
                     load_hook = hooks[defaultdict]
                     return DefaultDictParser(
@@ -396,6 +438,19 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
                         cls.get_parser_for_annotation
                     )
 
+                elif base_type in (abc.Sequence, abc.MutableSequence, abc.Collection):
+                    load_hook = cls.load_to_iterable
+                    # desired (non-generic) origin type
+                    desired_type = tuple if base_type is abc.Sequence else list
+                    # Re-map to desired type, e.g. `Sequence[int]` -> `tuple[int]`
+                    ann_type = desired_type[ann_type] if (
+                        ann_type := get_args(ann_type)[0]) else desired_type
+
+                    return IterableParser(
+                        base_cls, extras, ann_type, load_hook,
+                        cls.get_parser_for_annotation
+                    )
+
                 else:
                     load_hook = hooks.get(base_type)
 
@@ -478,6 +533,7 @@ def setup_default_loader(cls=LoadMixin):
     cls.register_load_hook(defaultdict, cls.load_to_defaultdict)
     cls.register_load_hook(dict, cls.load_to_dict)
     cls.register_load_hook(Decimal, cls.load_to_decimal)
+    cls.register_load_hook(Path, cls.load_to_path)
     # Dates and times
     cls.register_load_hook(datetime, cls.load_to_datetime)
     cls.register_load_hook(time, cls.load_to_time)
@@ -485,80 +541,20 @@ def setup_default_loader(cls=LoadMixin):
     cls.register_load_hook(timedelta, cls.load_to_timedelta)
 
 
-def get_loader(class_or_instance=None, create=True) -> Type[LoadMixin]:
-    """
-    Get the loader for the class, using the following logic:
-
-        * Return the class if it's already a sub-class of :class:`LoadMixin`
-        * If `create` is enabled (which is the default), a new sub-class of
-          :class:`LoadMixin` for the class will be generated and cached on the
-          initial run.
-        * Otherwise, we will return the base loader, :class:`LoadMixin`, which
-          can potentially be shared by more than one dataclass.
-
-    """
-    try:
-        return dataclass_to_loader(class_or_instance)
-
-    except KeyError:
-
-        if hasattr(class_or_instance, _LOAD_HOOKS):
-            return set_class_loader(class_or_instance, class_or_instance)
-
-        elif create:
-            cls_loader = create_new_class(class_or_instance, (LoadMixin, ))
-            return set_class_loader(class_or_instance, cls_loader)
-
-        return set_class_loader(class_or_instance, LoadMixin)
-
-
-def fromdict(cls: Type[T], d: JSONObject) -> T:
-    """
-    Converts a Python dictionary object to a dataclass instance.
-
-    Iterates over each dataclass field recursively; lists, dicts, and nested
-    dataclasses will likewise be initialized as expected.
-
-    When directly invoking this function, an optional Meta configuration for
-    the dataclass can be specified via ``LoadMeta``; by default, this will
-    apply recursively to any nested dataclasses. Here's a sample usage of this
-    below::
-
-        >>> LoadMeta(key_transform='CAMEL').bind_to(MyClass)
-        >>> fromdict(MyClass, {"myStr": "value"})
-
-    """
-    try:
-        load = _CLASS_TO_LOAD_FUNC[cls]
-    except KeyError:
-        load = load_func_for_dataclass(cls)
-
-    return load(d)
-
-
-def fromlist(cls: Type[T], list_of_dict: List[JSONObject]) -> List[T]:
-    """
-    Converts a Python list object to a list of dataclass instances.
-
-    Iterates over each dataclass field recursively; lists, dicts, and nested
-    dataclasses will likewise be initialized as expected.
-
-    """
-    try:
-        load = _CLASS_TO_LOAD_FUNC[cls]
-    except KeyError:
-        load = load_func_for_dataclass(cls)
-
-    return [load(d) for d in list_of_dict]
-
-
 def load_func_for_dataclass(
         cls: Type[T],
         is_main_class: bool = True,
-        config: Optional[META] = None) -> Callable[[JSONObject], T]:
+        config: Optional[META] = None,
+        loader_cls=LoadMixin,
+) -> Callable[[JSONObject], T]:
+
+    # TODO dynamically generate for multiple nested classes at once
+
+    # Tuple describing the fields of this dataclass.
+    cls_fields = dataclass_fields(cls)
 
     # Get the loader for the class, or create a new one as needed.
-    cls_loader = get_loader(cls)
+    cls_loader = get_loader(cls, base_cls=loader_cls, v1=False)
 
     # Get the meta config for the class, or the default config otherwise.
     meta = get_meta(cls)
@@ -569,148 +565,221 @@ def load_func_for_dataclass(
         if meta.recursive and meta is not AbstractMeta:
             config = meta
 
-    else:  # we are being run for a nested dataclass
-        if config:
-            # we want to apply the meta config from the main dataclass
-            # recursively.
-            meta = meta | config
-            meta.bind_to(cls, is_default=False)
+    # we are being run for a nested dataclass
+    elif config:
+        # we want to apply the meta config from the main dataclass
+        # recursively.
+        meta = meta | config
+        meta.bind_to(cls, is_default=False)
 
     # This contains a mapping of the original field name to the parser for its
     # annotated type; the item lookup *can* be case-insensitive.
-    field_to_parser = dataclass_field_to_load_parser(cls_loader, cls, config)
+    try:
+        field_to_parser = dataclass_field_to_load_parser(cls_loader, cls, config)
+    except RecursionError:
+        if meta.recursive_classes:
+            # recursion-safe loader is already in use; something else must have gone wrong
+            raise
+        else:
+            raise RecursiveClassError(cls) from None
 
     # A cached mapping of each key in a JSON or dictionary object to the
     # resolved dataclass field name; useful so we don't need to do a case
     # transformation (via regex) each time.
-    json_to_dataclass_field = json_field_to_dataclass_field(cls)
+    json_to_field = json_field_to_dataclass_field(cls)
 
-    def cls_fromdict(o: JSONObject, *_):
-        """
-        De-serialize a dictionary `o` to an instance of a dataclass `cls`.
-        """
+    field_to_path = dataclass_field_to_json_path(cls)
+    num_paths = len(field_to_path)
+    has_json_paths = True if num_paths else False
+
+    catch_all_field = json_to_field.get(CATCH_ALL)
+    has_catch_all = catch_all_field is not None
+
+    # Fix for using `auto_assign_tags` and `raise_on_unknown_json_key` together
+    # See https://github.com/rnag/dataclass-wizard/issues/137
+    has_tag_assigned = meta.tag is not None
+    if (has_tag_assigned and
+            # Ensure `tag_key` isn't a dataclass field before assigning an
+            # `ExplicitNull`, as assigning it directly can cause issues.
+            # See https://github.com/rnag/dataclass-wizard/issues/148
+            meta.tag_key not in field_to_parser):
+        json_to_field[meta.tag_key] = ExplicitNull
+
+    _locals = {
+        'cls': cls,
+        'py_case': cls_loader.transform_json_field,
+        'field_to_parser': field_to_parser,
+        'json_to_field': json_to_field,
+        'ExplicitNull': ExplicitNull,
+    }
+
+    _globals = {
+        'cls_fields': cls_fields,
+        'LOG': LOG,
+        'MissingData': MissingData,
+        'MissingFields': MissingFields,
+    }
+
+    # Initialize the FuncBuilder
+    fn_gen = FunctionBuilder()
+
+    if has_json_paths:
+        loop_over_o = num_paths != len(dataclass_init_fields(cls))
+        _locals['safe_get'] = safe_get
+    else:
+        loop_over_o = True
+
+    with fn_gen.function('cls_fromdict', ['o'], MISSING, _locals):
+
+        _pre_from_dict_method = getattr(cls, '_pre_from_dict', None)
+        if _pre_from_dict_method is not None:
+            _locals['__pre_from_dict__'] = _pre_from_dict_method
+            fn_gen.add_line('o = __pre_from_dict__(o)')
 
         # Need to create a separate dictionary to copy over the constructor
         # args, as we don't want to mutate the original dictionary object.
-        cls_kwargs = {}
+        fn_gen.add_line('init_kwargs = {}')
+        if has_catch_all:
+            fn_gen.add_line('catch_all = {}')
 
-        # This try-block is here in case the object `o` is None.
-        try:
-            # Loop over the dictionary object
-            for json_key in o:
+        if has_json_paths:
+
+            with fn_gen.try_():
+                field_to_default = dataclass_field_to_default(cls)
+                for field, path in field_to_path.items():
+                    if field in field_to_default:
+                        default_value = f'_default_{field}'
+                        _locals[default_value] = field_to_default[field]
+                        extra_args = f', {default_value}'
+                    else:
+                        extra_args = ''
+                    fn_gen.add_line(f'field={field!r}; init_kwargs[field] = field_to_parser[field](safe_get(o, {path!r}{extra_args}))')
+
+            with fn_gen.except_(ParseError, 'e'):
+                # We run into a parsing error while loading the field value;
+                # Add additional info on the Exception object before re-raising it.
+                fn_gen.add_line("e.class_name, e.field_name, e.json_object, e.fields = cls, field, o, cls_fields")
+                fn_gen.add_line("raise")
+
+        if loop_over_o:
+            # This try-block is here in case the object `o` is None.
+            with fn_gen.try_():
+                # Loop over the dictionary object
+                with fn_gen.for_('json_key in o'):
+
+                    with fn_gen.try_():
+                        # Get the resolved dataclass field name
+                        fn_gen.add_line("field = json_to_field[json_key]")
+
+                    with fn_gen.except_(KeyError):
+                        fn_gen.add_line('# Lookup Field for JSON Key')
+                        # Determines the dataclass field which a JSON key should map to.
+                        # Note this logic only runs the initial time, i.e. the first time
+                        # we encounter the key in a JSON object.
+                        #
+                        # :raises UnknownKeysError: If there is no resolved field name for the
+                        #   JSON key, and`raise_on_unknown_json_key` is enabled in the Meta
+                        #   config for the class.
+
+                        # Short path: an identical-cased field name exists for the JSON key
+                        with fn_gen.if_('json_key in field_to_parser'):
+                            fn_gen.add_line("field = json_to_field[json_key] = json_key")
+
+                        with fn_gen.else_():
+                            # Transform JSON field name (typically camel-cased) to the
+                            # snake-cased variant which is convention in Python.
+                            fn_gen.add_line("py_field = py_case(json_key)")
+
+                            with fn_gen.try_():
+                                # Do a case-insensitive lookup of the dataclass field, and
+                                # cache the mapping, so we have it for next time
+                                fn_gen.add_line("field "
+                                                "= json_to_field[json_key] "
+                                                "= field_to_parser.get_key(py_field)")
+
+                            with fn_gen.except_(KeyError):
+                                # Else, we see an unknown field in the dictionary object
+                                fn_gen.add_line("field = json_to_field[json_key] = ExplicitNull")
+                                fn_gen.add_line("LOG.warning('JSON field %r missing from dataclass schema, "
+                                                "class=%r, parsed field=%r',json_key,cls,py_field)")
+
+                                # Raise an error here (if needed)
+                                if meta.raise_on_unknown_json_key:
+                                    _globals['UnknownKeysError'] = UnknownKeysError
+                                    fn_gen.add_line("raise UnknownKeysError(json_key, o, cls, cls_fields) from None")
 
-                # Get the resolved dataclass field name
-                try:
-                    field_name = json_to_dataclass_field[json_key]
                     # Exclude JSON keys that don't map to any fields.
-                    if field_name is ExplicitNull:
-                        continue
+                    with fn_gen.if_('field is not ExplicitNull'):
 
-                except KeyError:
-                    try:
-                        field_name = lookup_field_for_json_key(o, json_key)
-                    except LookupError:
-                        continue
+                        with fn_gen.try_():
+                            # Note: pass the original cased field to the class constructor;
+                            # don't use the lowercase result from `py_case`
+                            fn_gen.add_line("init_kwargs[field] = field_to_parser[field](o[json_key])")
 
-                try:
-                    # Note: pass the original cased field to the class
-                    # constructor; don't use the lowercase result from
-                    # `transform_json_field`
-                    cls_kwargs[field_name] = field_to_parser[field_name](
-                        o[json_key])
+                        with fn_gen.except_(ParseError, 'e'):
+                            # We run into a parsing error while loading the field value;
+                            # Add additional info on the Exception object before re-raising it.
+                            #
+                            # First confirm these values are not already set by an
+                            # inner dataclass. If so, it likely makes it easier to
+                            # debug the cause. Note that this should already be
+                            # handled by the `setter` methods.
+                            fn_gen.add_line("e.class_name, e.field_name, e.json_object = cls, field, o")
+                            fn_gen.add_line("raise")
 
-                except ParseError as e:
-                    # We run into a parsing error while loading the field
-                    # value; Add additional info on the Exception object
-                    # before re-raising it.
-                    #
-                    # First confirm these values are not already set by an
-                    # inner dataclass. If so, it likely makes it easier to
-                    # debug the cause. Note that this should already be
-                    # handled by the `setter` methods.
-                    e.class_name = cls
-                    e.field_name = field_name
-                    e.json_object = o
-                    raise
+                    if has_catch_all:
+                        line = 'catch_all[json_key] = o[json_key]'
+                        if has_tag_assigned:
+                            with fn_gen.elif_(f'json_key != {meta.tag_key!r}'):
+                                fn_gen.add_line(line)
+                        else:
+                            with fn_gen.else_():
+                                fn_gen.add_line(line)
 
-        except TypeError:
-            # If the object `o` is None, then raise an error with the relevant
-            # info included.
-            if o is None:
-                raise MissingData(cls) from None
+            with fn_gen.except_(TypeError):
+                # If the object `o` is None, then raise an error with
+                # the relevant info included.
+                with fn_gen.if_('o is None'):
+                    fn_gen.add_line("raise MissingData(cls) from None")
 
-            # Check if the object `o` is some other type than what we expect -
-            # for example, we could be passed in a `list` type instead.
-            if not isinstance(o, dict):
-                e = TypeError('Incorrect type for field')
-                raise ParseError(
-                    e, o, dict, cls,
-                    desired_type=dict
-                ) from None
+                # Check if the object `o` is some other type than what we expect -
+                # for example, we could be passed in a `list` type instead.
+                with fn_gen.if_('not isinstance(o, dict)'):
+                    fn_gen.add_line("e = TypeError('Incorrect type for field')")
+                    fn_gen.add_line("raise ParseError(e, o, dict, cls, desired_type=dict) from None")
 
-            #  Else, just re-raise the error.
-            raise
+                # Else, just re-raise the error.
+                fn_gen.add_line("raise")
 
-        # Now pass the arguments to the constructor method, and return the new
-        # dataclass instance. If there are any missing fields, we raise them
-        # here.
+        if has_catch_all:
+            if catch_all_field.endswith('?'):  # Default value
+                with fn_gen.if_('catch_all'):
+                    fn_gen.add_line(f'init_kwargs[{catch_all_field.rstrip("?")!r}] = catch_all')
+            else:
+                fn_gen.add_line(f'init_kwargs[{catch_all_field!r}] = catch_all')
 
-        try:
-            return cls(**cls_kwargs)
+        # Now pass the arguments to the constructor method, and return
+        # the new dataclass instance. If there are any missing fields,
+        # we raise them here.
 
-        except TypeError as e:
-            raise MissingFields(
-                e, o, cls, cls_kwargs, dataclass_fields(cls)
-            ) from None
+        with fn_gen.try_():
+            fn_gen.add_line("return cls(**init_kwargs)")
 
-    def lookup_field_for_json_key(o: JSONObject, json_field: str):
-        """
-        Determines the dataclass field which a JSON key should map to. Note
-        this only runs the initial time, i.e. the first time we encounter the
-        key in a JSON object.
+        with fn_gen.except_(TypeError, 'e'):
+            fn_gen.add_line("raise MissingFields(e, o, cls, cls_fields, init_kwargs) from None")
 
-        :raises LookupError: If there no resolved field name for the JSON key.
-        :raises UnknownJSONKey: If there is no resolved field name for the
-          JSON key, and`raise_on_unknown_json_key` is enabled in the Meta
-          config for the class.
-        """
+    functions = fn_gen.create_functions(_globals)
 
-        # Short path: an identical-cased field name exists for the JSON key
-        if json_field in field_to_parser:
-            json_to_dataclass_field[json_field] = json_field
-            return json_field
-
-        # Transform JSON field name (typically camel-cased) to the
-        # snake-cased variant which is convention in Python.
-        transformed_field = cls_loader.transform_json_field(json_field)
-
-        try:
-            # Do a case-insensitive lookup of the dataclass field, and
-            # cache the mapping, so we have it for next time
-            field_name = field_to_parser.get_key(transformed_field)
-            json_to_dataclass_field[json_field] = field_name
-
-        except KeyError:
-            # Else, we see an unknown field in the dictionary object
-            json_to_dataclass_field[json_field] = ExplicitNull
-            LOG.warning(
-                'JSON field %r missing from dataclass schema, '
-                'class=%r, parsed field=%r',
-                json_field, get_class_name(cls), transformed_field)
-
-            # Raise an error here (if needed)
-            if meta.raise_on_unknown_json_key:
-                cls_fields = dataclass_fields(cls)
-                e = UnknownJSONKey(json_field, o, cls, cls_fields)
-                raise e from None
-
-            raise LookupError
-
-        return field_name
+    cls_fromdict = functions['cls_fromdict']
 
     # Save the load function for the main dataclass, so we don't need to run
     # this logic each time.
     if is_main_class:
-        _CLASS_TO_LOAD_FUNC[cls] = cls_fromdict
+        # Check if the class has a `from_dict`, and it's
+        # a class method bound to `fromdict`.
+        if ((from_dict := getattr(cls, 'from_dict', None)) is not None
+                and getattr(from_dict, '__func__', None) is fromdict):
+            _set_new_attribute(cls, 'from_dict', cls_fromdict)
+        CLASS_TO_LOAD_FUNC[cls] = cls_fromdict
 
     return cls_fromdict
