@@ -1,6 +1,7 @@
 __all__ = ['IdentityParser',
            'SingleArgParser',
            'Parser',
+           'RecursionSafeParser',
            'PatternedDTParser',
            'LiteralParser',
            'UnionParser',
@@ -19,28 +20,29 @@ from typing import (
     Type, Any, Optional, Tuple, Dict, Iterable, Callable, List
 )
 
-from .abstractions import AbstractParser, FieldToParser
+from .abstractions import AbstractParser
 from .bases import AbstractMeta
 from .class_helper import get_meta, _META
 from .constants import TAG
 from .errors import ParseError
-from .models import _PatternedDT, Extras
+from .models import PatternedDT, Extras
 from .type_def import (
     FrozenKeys, NoneType, DefFactory,
-    T, M, S, DD, LSQ, N, NT
+    T, M, S, DD, LSQ, N, NT, DT
 )
 from .utils.typing_compat import (
-    get_origin, get_args, get_named_tuple_field_types,
+    get_origin, get_args,
     get_keys_for_typed_dict, eval_forward_ref_if_needed)
 
 
 # Type defs
 GetParserType = Callable[[Type[T], Type, Extras], AbstractParser]
+LoadHookType = Callable[[Any], T]
 TupleOfParsers = Tuple[AbstractParser, ...]
 
 
 @dataclass
-class IdentityParser(AbstractParser):
+class IdentityParser(AbstractParser[Type[T], T]):
     __slots__ = ()
 
     def __call__(self, o: Any) -> T:
@@ -48,10 +50,10 @@ class IdentityParser(AbstractParser):
 
 
 @dataclass
-class SingleArgParser(AbstractParser):
+class SingleArgParser(AbstractParser[Type[T], T]):
     __slots__ = ('hook', )
 
-    hook: Callable[[Any], T]
+    hook: LoadHookType
 
     # noinspection PyDataclass
     def __post_init__(self, *_):
@@ -63,20 +65,58 @@ class SingleArgParser(AbstractParser):
 
 
 @dataclass
-class Parser(AbstractParser):
+class Parser(AbstractParser[T, T]):
     __slots__ = ('hook', )
 
-    hook: Callable[[Any, Type[T]], T]
+    hook: Callable[[Any, type[T]], T]
 
     def __call__(self, o: Any) -> T:
         return self.hook(o, self.base_type)
 
 
 @dataclass
-class LiteralParser(AbstractParser):
+class RecursionSafeParser(AbstractParser):
+    """
+    Parser to handle cyclic or self-referential dataclasses.
+
+    For example::
+
+        @dataclass
+        class A:
+            a: A | None = None
+
+        instance = fromdict(A, {'a': {'a': {'a': None}}})
+    """
+    __slots__ = ('extras', 'hook')
+
+    extras: Extras
+    hook: Optional[LoadHookType]
+
+    def load_hook_func(self) -> LoadHookType:
+        from .loaders import load_func_for_dataclass
+
+        return load_func_for_dataclass(
+            self.base_type,
+            is_main_class=False,
+            config=self.extras['config']
+        )
+
+    # TODO: decorating `load_hook_func` with `@cached_property` could
+    #   be an alternate, bit cleaner approach.
+    def __call__(self, o: Any) -> T:
+        load_hook = self.hook
+
+        if load_hook is None:
+            load_hook = self.hook = self.load_hook_func()
+
+        return load_hook(o)
+
+
+@dataclass
+class LiteralParser(AbstractParser[M, M]):
     __slots__ = ('value_to_type', )
 
-    base_type: Type[M]
+    base_type: type[M]
 
     # noinspection PyDataclass
     def __post_init__(self, *_):
@@ -84,18 +124,26 @@ class LiteralParser(AbstractParser):
             val: type(val) for val in get_args(self.base_type)
         }
 
-    def __call__(self, o: Any):
+    def __contains__(self, item) -> bool:
+        """
+        Return true if the LiteralParser is expected to handle the specified item
+        type. Checks that the item is incorporated in the given expected values of
+        the Literal.
+        """
+        return item in self.value_to_type
+
+    def __call__(self, o: Any) -> M:
         """
         Checks for Literal equivalence, as mentioned here:
           https://www.python.org/dev/peps/pep-0586/#equivalence-of-two-literals
 
         """
         try:
-            type_does_not_match = type(o) != self.value_to_type[o]
+            type_does_not_match = type(o) is not self.value_to_type[o]
 
         except KeyError:
             # No such Literal with the value of `o`
-            e = ValueError('Value not in expected Literal values')
+            e: Exception = ValueError('Value not in expected Literal values')
             raise ParseError(
                 e, o, self.base_type,
                 allowed_values=list(self.value_to_type))
@@ -119,21 +167,21 @@ class LiteralParser(AbstractParser):
 
 
 @dataclass
-class PatternedDTParser(AbstractParser):
+class PatternedDTParser(AbstractParser[PatternedDT, DT]):
     __slots__ = ('hook', )
 
-    base_type: _PatternedDT
+    base_type: PatternedDT
 
     # noinspection PyDataclass
     def __post_init__(self, _cls: Type, extras: Extras, *_):
-        if not isinstance(self.base_type, _PatternedDT):
+        if not isinstance(self.base_type, PatternedDT):
             dt_cls = self.base_type
             self.base_type = extras['pattern']
             self.base_type.cls = dt_cls
 
         self.hook = self.base_type.get_transform_func()
 
-    def __call__(self, date_string: str):
+    def __call__(self, date_string: str) -> DT:
         try:
             return self.hook(date_string)
         except ValueError as e:
@@ -144,7 +192,7 @@ class PatternedDTParser(AbstractParser):
 
 
 @dataclass
-class OptionalParser(AbstractParser):
+class OptionalParser(AbstractParser[T, Optional[T]]):
     __slots__ = ('parser', )
 
     get_parser: InitVar[GetParserType]
@@ -153,7 +201,10 @@ class OptionalParser(AbstractParser):
                       extras: Extras,
                       get_parser: GetParserType):
 
-        self.parser: AbstractParser = get_parser(self.base_type, cls, extras)
+        self.parser: AbstractParser = getattr(
+            p := get_parser(self.base_type, cls, extras),
+            '__call__', p
+        )
 
     def __contains__(self, item):
         """Check if parser is expected to handle the specified item type."""
@@ -162,7 +213,7 @@ class OptionalParser(AbstractParser):
 
         return super().__contains__(item)
 
-    def __call__(self, o: Any):
+    def __call__(self, o: Any) -> Optional[T]:
         if o is None:
             return o
 
@@ -170,7 +221,7 @@ class OptionalParser(AbstractParser):
 
 
 @dataclass
-class UnionParser(AbstractParser):
+class UnionParser(AbstractParser[Tuple[Type[T], ...], Optional[T]]):
     __slots__ = ('parsers', 'tag_to_parser', 'tag_key')
 
     base_type: Tuple[Type[T], ...]
@@ -190,38 +241,45 @@ class UnionParser(AbstractParser):
             self.tag_key = TAG
             auto_assign_tags = False
 
-        self.parsers = tuple(
-            get_parser(t, cls, extras) for t in self.base_type
-            if t is not NoneType)
-
+        parsers_list = []
         self.tag_to_parser = {}
+
         for t in self.base_type:
             t = eval_forward_ref_if_needed(t, cls)
-            if is_dataclass(t):
-                meta = get_meta(t)
-                tag = meta.tag
-                if not tag and (auto_assign_tags or meta.auto_assign_tags):
-                    cls_name = t.__name__
-                    tag = cls_name
-                    # We don't want to mutate the base Meta class here
-                    if meta is AbstractMeta:
-                        from .bases_meta import BaseJSONWizardMeta
-                        cls_dict = {'__slots__': (), 'tag': tag}
-                        meta = type(cls_name + 'Meta', (BaseJSONWizardMeta, ), cls_dict)
-                        _META[t] = meta
-                    else:
-                        meta.tag = cls_name
-                if tag:
-                    # TODO see if we can use a mapping of dataclass type to
-                    #   load func (maybe one passed in to __post_init__),
-                    #   rather than generating one on the fly like this.
-                    self.tag_to_parser[tag] = get_parser(t, cls, extras)
+            if t is not NoneType:
+                parser = get_parser(t, cls, extras)
+
+                if isinstance(parser, AbstractParser):
+                    parsers_list.append(parser)
+
+                elif is_dataclass(t):
+                    meta = get_meta(t)
+                    tag = meta.tag
+                    if not tag and (auto_assign_tags or meta.auto_assign_tags):
+                        cls_name = t.__name__
+                        tag = cls_name
+                        # We don't want to mutate the base Meta class here
+                        if meta is AbstractMeta:
+                            from .bases_meta import BaseJSONWizardMeta
+                            cls_dict = {'__slots__': (), 'tag': tag}
+                            # noinspection PyTypeChecker
+                            meta: type[M] = type(cls_name + 'Meta', (BaseJSONWizardMeta, ), cls_dict)
+                            _META[t] = meta
+                        else:
+                            meta.tag = cls_name
+                    if tag:
+                        # TODO see if we can use a mapping of dataclass type to
+                        #   load func (maybe one passed in to __post_init__),
+                        #   rather than generating one on the fly like this.
+                        self.tag_to_parser[tag] = parser
+
+        self.parsers = tuple(parsers_list)
 
     def __contains__(self, item):
         """Check if parser is expected to handle the specified item type."""
         return type(item) in self.base_type
 
-    def __call__(self, o: Any):
+    def __call__(self, o: Any) -> Optional[T]:
         if o is None:
             return o
 
@@ -255,7 +313,7 @@ class UnionParser(AbstractParser):
 
 
 @dataclass
-class IterableParser(AbstractParser):
+class IterableParser(AbstractParser[Type[LSQ], LSQ]):
     """
     Parser for a :class:`list`, :class:`set`, :class:`frozenset`,
     :class:`deque`, or a subclass of either type.
@@ -282,7 +340,9 @@ class IterableParser(AbstractParser):
         #   ex. `List[str]` -> `list`
         self.base_type = get_origin(self.base_type)
 
-        self.elem_parser = get_parser(elem_type, cls, extras)
+        self.elem_parser = getattr(
+            p := get_parser(elem_type, cls, extras), '__call__', p,
+        )
 
     def __call__(self, o: Iterable) -> LSQ:
         """
@@ -290,21 +350,11 @@ class IterableParser(AbstractParser):
 
         See the declaration of :var:`LSQ` for more info.
         """
-        try:
-            return self.hook(o, self.base_type, self.elem_parser)
-        # TODO
-        except Exception:
-            if not isinstance(o, self.base_type):
-                e = TypeError('Incorrect type for field')
-                raise ParseError(
-                    e, o, self.base_type,
-                    desired_type=self.base_type)
-            else:
-                raise
+        return self.hook(o, self.base_type, self.elem_parser)
 
 
 @dataclass
-class TupleParser(AbstractParser):
+class TupleParser(AbstractParser[Type[S], S]):
     """
     Parser for subscripted and un-subscripted :class:`Tuple`'s.
 
@@ -314,12 +364,13 @@ class TupleParser(AbstractParser):
     __slots__ = ('hook',
                  'elem_parsers',
                  'total_count',
-                 'required_count')
+                 'required_count',
+                 'elem_types')
 
     # Base type of the object which is instantiable
     #   ex. `Tuple[bool, int]` -> `tuple`
     base_type: Type[S]
-    hook: Callable[[Any, Type[S], TupleOfParsers], S]
+    hook: Callable[[Any, Type[S], Optional[TupleOfParsers]], S]
     get_parser: InitVar[GetParserType]
 
     def __post_init__(self, cls: Type,
@@ -328,24 +379,25 @@ class TupleParser(AbstractParser):
 
         # Get the subscripted values
         #   ex. `Tuple[bool, int]` -> (bool, int)
-        elem_types = get_args(self.base_type)
+        self.elem_types = elem_types = get_args(self.base_type)
         self.base_type = get_origin(self.base_type)
         # A collection with a parser for each type argument
-        self.elem_parsers = tuple(get_parser(t, cls, extras)
-                                  for t in elem_types)
+        elem_parsers = tuple(get_parser(t, cls, extras)
+                             for t in elem_types)
         # Total count is generally the number of type arguments to `Tuple`, but
         # can be `Infinity` when a `Tuple` appears in its un-subscripted form.
-        self.total_count: N = len(self.elem_parsers) or float('inf')
+        self.total_count: N = len(elem_parsers) or float('inf')
         # Minimum number of *required* type arguments
         #   Check for the count of parsers which don't handle `NoneType` -
         #   this should exclude the parsers for `Optional` or `Union` types
         #   that have `None` in the list of args.
-        self.required_count: int = len(tuple(p for p in self.elem_parsers
-                                             if None not in p))
-        if not self.elem_parsers:
-            self.elem_parsers = None
+        self.required_count: int = len(tuple(p for p in elem_parsers
+                                             if not isinstance(p, AbstractParser)
+                                             or None not in p))
 
-    def __call__(self, o: M) -> M:
+        self.elem_parsers = elem_parsers or None
+
+    def __call__(self, o: S) -> S:
         """
         Load an object `o` into a new object of type `base_type` (generally a
         :class:`tuple` or a sub-class of one)
@@ -357,10 +409,15 @@ class TupleParser(AbstractParser):
             if self.required_count != self.total_count:
                 desired_count = f'{self.required_count} - {self.total_count}'
             else:
-                desired_count = self.total_count
+                desired_count = str(self.total_count)
+
+            # self.elem_parsers can be None at this moment
+            elem_parsers_types = [getattr(p, 'base_type', tp) for p, tp in
+                                  zip(self.elem_parsers, self.elem_types)] \
+                if self.elem_parsers else self.elem_types
 
             raise ParseError(
-                e, o, [p.base_type for p in self.elem_parsers],
+                e, o, elem_parsers_types,
                 desired_count=desired_count,
                 actual_count=len(o))
 
@@ -401,10 +458,7 @@ class VariadicTupleParser(TupleParser):
         # Total count should be `Infinity` here, since the variadic form
         # accepts any number of possible arguments.
         self.total_count: N = float('inf')
-        # Check for the count of parsers which don't handle `NoneType` - this
-        # should exclude the parsers for `Union` types that have `None` in the
-        # list of args.
-        self.required_count = 0 if None in self.first_elem_parser[0] else 1
+        self.required_count = 0
 
     def __call__(self, o: M) -> M:
         """
@@ -416,14 +470,13 @@ class VariadicTupleParser(TupleParser):
 
 
 @dataclass
-class NamedTupleParser(AbstractParser):
+class NamedTupleParser(AbstractParser[tuple, NT]):
     __slots__ = ('hook',
                  'field_to_parser',
                  'field_parsers')
 
-    base_type: Type[S]
     hook: Callable[
-        [Any, Type[NT], Optional[FieldToParser], List[AbstractParser]],
+        [Any, type[tuple], Optional['FieldToParser'], List[AbstractParser]],
         NT
     ]
     get_parser: InitVar[GetParserType]
@@ -433,18 +486,16 @@ class NamedTupleParser(AbstractParser):
                       get_parser: GetParserType):
 
         # Get the field annotations for the `NamedTuple` type
-        type_anns: Dict[str, Type[T]] = get_named_tuple_field_types(
-            self.base_type
-        )
+        type_anns: Dict[str, type[T]] = self.base_type.__annotations__
 
-        self.field_to_parser: Optional[FieldToParser] = {
-            f: get_parser(ftype, cls, extras)
+        self.field_to_parser: Optional['FieldToParser'] = {
+            f: getattr(p := get_parser(ftype, cls, extras), '__call__', p)
             for f, ftype in type_anns.items()
         }
 
         self.field_parsers = list(self.field_to_parser.values())
 
-    def __call__(self, o: Any):
+    def __call__(self, o: Any) -> NT:
         """
         Load a dictionary or list to a `NamedTuple` sub-class (or an
         un-annotated `namedtuple`)
@@ -454,23 +505,22 @@ class NamedTupleParser(AbstractParser):
 
 
 @dataclass
-class NamedTupleUntypedParser(AbstractParser):
+class NamedTupleUntypedParser(AbstractParser[tuple, NT]):
     __slots__ = ('hook',
                  'dict_parser',
                  'list_parser')
 
-    base_type: Type[S]
-    hook: Callable[[Any, Type[NT], AbstractParser, AbstractParser], NT]
+    hook: Callable[[Any, Type[tuple], AbstractParser, AbstractParser], NT]
     get_parser: InitVar[GetParserType]
 
     def __post_init__(self, cls: Type,
                       extras: Extras,
                       get_parser: GetParserType):
 
-        self.dict_parser = get_parser(dict, cls, extras)
-        self.list_parser = get_parser(list, cls, extras)
+        self.dict_parser = get_parser(dict, cls, extras).__call__
+        self.list_parser = get_parser(list, cls, extras).__call__
 
-    def __call__(self, o: Any):
+    def __call__(self, o: Any) -> NT:
         """
         Load a dictionary or list to a `NamedTuple` sub-class (or an
         un-annotated `namedtuple`)
@@ -480,10 +530,11 @@ class NamedTupleUntypedParser(AbstractParser):
 
 
 @dataclass
-class MappingParser(AbstractParser):
+class MappingParser(AbstractParser[Type[M], M]):
     __slots__ = ('hook',
                  'key_parser',
-                 'val_parser')
+                 'val_parser',
+                 'val_type')
 
     base_type: Type[M]
     hook: Callable[[Any, Type[M], AbstractParser, AbstractParser], M]
@@ -500,16 +551,19 @@ class MappingParser(AbstractParser):
         # Base type of the object which is instantiable
         #   ex. `Dict[str, Any]` -> `dict`
         self.base_type: Type[M] = get_origin(self.base_type)
+        self.val_type = val_type
 
-        self.key_parser = get_parser(key_type, cls, extras)
-        self.val_parser = get_parser(val_type, cls, extras)
+        val_parser = get_parser(val_type, cls, extras)
+
+        self.key_parser = getattr(p := get_parser(key_type, cls, extras), '__call__', p)
+        self.val_parser = getattr(val_parser, '__call__', val_parser)
 
     def __call__(self, o: M) -> M:
         return self.hook(o, self.base_type, self.key_parser, self.val_parser)
 
 
 @dataclass
-class DefaultDictParser(MappingParser):
+class DefaultDictParser(MappingParser[DD]):
     __slots__ = ('default_factory', )
 
     # Override the type annotations here
@@ -523,30 +577,32 @@ class DefaultDictParser(MappingParser):
         super().__post_init__(cls, extras, get_parser)
 
         # The default factory argument to pass to the `defaultdict` subclass
-        self.default_factory: DefFactory = self.val_parser.base_type
+        val_type = self.val_type
+        val_base_type = getattr(val_type, '__origin__', val_type)
+        self.default_factory: DefFactory = val_base_type
 
-    def __call__(self, o: M) -> M:
+    def __call__(self, o: DD) -> DD:
         return self.hook(o, self.base_type, self.default_factory,
                          self.key_parser, self.val_parser)
 
 
 @dataclass
-class TypedDictParser(AbstractParser):
+class TypedDictParser(AbstractParser[Type[M], M]):
     __slots__ = ('hook',
                  'key_to_parser',
                  'required_keys',
                  'optional_keys')
 
-    base_type: Type[S]
-    hook: Callable[[Any, Type[M], FieldToParser, FrozenKeys, FrozenKeys], M]
+    base_type: Type[M]
+    hook: Callable[[Any, Type[M], 'FieldToParser', FrozenKeys, FrozenKeys], M]
     get_parser: InitVar[GetParserType]
 
     def __post_init__(self, cls: Type,
                       extras: Extras,
                       get_parser: GetParserType):
 
-        self.key_to_parser: FieldToParser = {
-            k: get_parser(v, cls, extras)
+        self.key_to_parser: 'FieldToParser' = {
+            k: getattr(p := get_parser(v, cls, extras), '__call__', p)
             for k, v in self.base_type.__annotations__.items()
         }
 
@@ -560,13 +616,13 @@ class TypedDictParser(AbstractParser):
                              self.required_keys, self.optional_keys)
 
         except KeyError as e:
-            e = KeyError(f'Missing required key: {e.args[0]}')
-            raise ParseError(e, o, self.base_type)
+            err: Exception = KeyError(f'Missing required key: {e.args[0]}')
+            raise ParseError(err, o, self.base_type)
 
         except Exception:
             if not isinstance(o, dict):
-                e = TypeError('Incorrect type for object')
+                err = TypeError('Incorrect type for object')
                 raise ParseError(
-                    e, o, self.base_type, desired_type=self.base_type)
+                    err, o, self.base_type, desired_type=self.base_type)
             else:
                 raise
