@@ -1,14 +1,14 @@
 import logging
 from dataclasses import dataclass, field
 from datetime import datetime, date
-from typing import Optional, List, Type
+from typing import Optional, List
 from unittest.mock import ANY
 
 import pytest
 from pytest_mock import MockerFixture
 
-from dataclass_wizard import JSONWizard
-from dataclass_wizard.bases import M
+from dataclass_wizard.bases import META
+from dataclass_wizard import JSONWizard, EnvWizard
 from dataclass_wizard.bases_meta import BaseJSONWizardMeta
 from dataclass_wizard.enums import LetterCase, DateTimeTo
 from dataclass_wizard.errors import ParseError
@@ -18,14 +18,9 @@ from dataclass_wizard.utils.type_conv import date_to_timestamp
 log = logging.getLogger(__name__)
 
 
-@pytest.fixture
-def mock_log(mocker: MockerFixture):
-    return mocker.patch('dataclass_wizard.bases_meta.LOG')
-
-
 @pytest.fixture
 def mock_meta_initializers(mocker: MockerFixture):
-    return mocker.patch('dataclass_wizard.bases_meta._META_INITIALIZER')
+    return mocker.patch('dataclass_wizard.bases_meta.META_INITIALIZER')
 
 
 @pytest.fixture
@@ -34,6 +29,12 @@ def mock_bind_to(mocker: MockerFixture):
         'dataclass_wizard.bases_meta.BaseJSONWizardMeta.bind_to')
 
 
+@pytest.fixture
+def mock_env_bind_to(mocker: MockerFixture):
+    return mocker.patch(
+        'dataclass_wizard.bases_meta.BaseEnvWizardMeta.bind_to')
+
+
 @pytest.fixture
 def mock_get_dumper(mocker: MockerFixture):
     return mocker.patch('dataclass_wizard.bases_meta.get_dumper')
@@ -56,7 +57,7 @@ def test_merge_meta_with_or():
         json_key_to_field = {'k2': 'v2'}
 
     # Merge the two Meta config together
-    merged_meta: M = A | B
+    merged_meta: META = A | B
 
     # Assert we are a subclass of A, which subclasses from `BaseJSONWizardMeta`
     assert issubclass(merged_meta, BaseJSONWizardMeta)
@@ -99,7 +100,7 @@ def test_merge_meta_with_and():
         json_key_to_field = {'k2': 'v2'}
 
     # Merge the two Meta config together
-    merged_meta: M = A & B
+    merged_meta: META = A & B
 
     # Assert we are a subclass of A, which subclasses from `BaseJSONWizardMeta`
     assert issubclass(merged_meta, BaseJSONWizardMeta)
@@ -150,7 +151,7 @@ def test_meta_initializer_runs_as_expected(mock_log):
         isActive: bool = False
         myDt: Optional[datetime] = None
 
-    mock_log.info.assert_called_once_with('DEBUG Mode is enabled')
+    assert 'DEBUG Mode is enabled' in mock_log.text
 
     string = """
     {
@@ -191,7 +192,7 @@ def test_meta_initializer_runs_as_expected(mock_log):
     assert d['my_dt'] == round(expected_dt.timestamp())
 
 
-def test_json_key_to_field_when_add_is_a_falsy_value(mock_log):
+def test_json_key_to_field_when_add_is_a_falsy_value():
     """
     The `json_key_to_field` attribute is specified when subclassing
     :class:`JSONWizard.Meta`, but the `__all__` field a falsy value.
@@ -213,7 +214,7 @@ def test_json_key_to_field_when_add_is_a_falsy_value(mock_log):
         myCustomStr: str
 
     # note: this is only expected to run at most once
-    # mock_log.info.assert_called_once_with('DEBUG Mode is enabled')
+    # assert 'DEBUG Mode is enabled' in mock_log.text
 
     string = """
     {
@@ -236,7 +237,7 @@ def test_json_key_to_field_when_add_is_a_falsy_value(mock_log):
     assert d['my_custom_str'] == "test that this is mapped to 'myCustomStr'"
 
 
-def test_meta_config_is_not_implicitly_shared_between_dataclasses(mock_log):
+def test_meta_config_is_not_implicitly_shared_between_dataclasses():
 
     @dataclass
     class MyFirstClass(JSONWizard):
@@ -323,6 +324,20 @@ def test_meta_initializer_is_called_when_meta_is_an_inner_class(
     mock_meta_initializers.__setitem__.assert_called_once()
 
 
+def test_env_meta_initializer_not_called_when_meta_is_not_an_inner_class(
+        mock_meta_initializers, mock_env_bind_to):
+    """
+    Meta Initializer `dict` should *not* be updated when `Meta` has no outer
+    class.
+    """
+
+    class _(EnvWizard.Meta):
+        debug_enabled = True
+
+    mock_meta_initializers.__setitem__.assert_not_called()
+    mock_env_bind_to.assert_called_once_with(ANY, create=False)
+
+
 def test_meta_initializer_not_called_when_meta_is_not_an_inner_class(
         mock_meta_initializers, mock_bind_to):
     """
@@ -337,8 +352,7 @@ def test_meta_initializer_not_called_when_meta_is_not_an_inner_class(
     mock_bind_to.assert_called_once_with(ANY, create=False)
 
 
-def test_meta_initializer_errors_when_key_transform_with_load_is_invalid(
-        mock_log):
+def test_meta_initializer_errors_when_key_transform_with_load_is_invalid():
     """
     Test when an invalid value for the ``key_transform_with_load`` attribute
     is specified when sub-classing from :class:`JSONWizard.Meta`.
@@ -355,8 +369,7 @@ def test_meta_initializer_errors_when_key_transform_with_load_is_invalid(
             list_of_int: List[int] = field(default_factory=list)
 
 
-def test_meta_initializer_errors_when_key_transform_with_dump_is_invalid(
-        mock_log):
+def test_meta_initializer_errors_when_key_transform_with_dump_is_invalid():
     """
     Test when an invalid value for the ``key_transform_with_dump`` attribute
     is specified when sub-classing from :class:`JSONWizard.Meta`.
@@ -373,8 +386,7 @@ def test_meta_initializer_errors_when_key_transform_with_dump_is_invalid(
             list_of_int: List[int] = field(default_factory=list)
 
 
-def test_meta_initializer_errors_when_marshal_date_time_as_is_invalid(
-        mock_log):
+def test_meta_initializer_errors_when_marshal_date_time_as_is_invalid():
     """
     Test when an invalid value for the ``marshal_date_time_as`` attribute
     is specified when sub-classing from :class:`JSONWizard.Meta`.
@@ -391,8 +403,7 @@ def test_meta_initializer_errors_when_marshal_date_time_as_is_invalid(
             list_of_int: List[int] = field(default_factory=list)
 
 
-def test_meta_initializer_is_noop_when_marshal_date_time_as_is_iso_format(
-        mock_log, mock_get_dumper):
+def test_meta_initializer_is_noop_when_marshal_date_time_as_is_iso_format(mock_get_dumper):
     """
     Test that it's a noop when the value for ``marshal_date_time_as``
     is `ISO_FORMAT`, which is the default conversion method for the dumper
